import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';
import 'package:get/get.dart';
import 'package:wowit/Controller/baseController.dart';
import 'package:wowit/Controller/connectController.dart';
import 'package:wowit/Widget/loadingWidget.dart';
import 'package:wowit/Widget/screenMessage.dart';
import 'package:wowit/common.dart';

abstract class BaseWidget<T extends BaseController> extends StatelessWidget {
  final bool global;
  final bool isComponent;
  final String tag;
  final T _baseController;

  const BaseWidget(
    T baseController, {
    this.isComponent = true,
    this.tag,
    this.global = true,
  }) : _baseController = baseController;

  T getController({String tag}) {
    if (global) {
      if (Get.isRegistered<T>(tag: tag) || Get.isPrepared<T>(tag: tag)) {
        return Get.find<T>(tag: tag);
      } else {
        return Get.put<T>(_baseController, tag: tag);
      }
    } else {
      return _baseController;
    }
  }

  T get controller => _baseController;

  @override
  String toString({DiagnosticLevel minLevel = DiagnosticLevel.info}) {
    return this.runtimeType.toString();
  }

  @override
  Widget build(BuildContext context) {
    return this._getWidget(context);
  }

  void initState(GetBuilderState<T> state) async {
    Get.log(
      '${this.toString()} initState',
      isError: false,
    );
    _baseController.isComponent = isComponent;
    MyConnectivity.instance.initialise(_baseController);
    if (Get.isPrepared<T>(tag: tag)) {
      Get.replace<T>(_baseController, tag: tag);
    } else {
      Get.lazyPut(() => _baseController);
    }
    if (Common.supperAdmin && !_baseController.isComponent) {
      SchedulerBinding.instance.addPostFrameCallback((_) {
        _baseController.messageIdBackUp = Common.messageIdBackUp;
      });
    }
  }

  void dispose(GetBuilderState<T> state) {
    Get.log(
      '${this.toString()} dispose',
      isError: false,
    );
    if (!global) {
      state.controller = null;
    }
  }

  void didUpdateWidget(GetBuilder oldWidget, GetBuilderState<T> state) {
    Get.log(
      '${this.toString()} didUpdateWidget',
      isError: false,
    );
  }

  void didChangeDependencies(GetBuilderState<T> state) {
    Get.log(
      '${this.toString()} didChangeDependencies',
      isError: false,
    );
  }

  Widget _getWidget(BuildContext context) {
    return GetBuilder<T>(
      global: global,
      init: _baseController,
      tag: tag,
      initState: initState,
      dispose: dispose,
      didUpdateWidget: didUpdateWidget,
      didChangeDependencies: didChangeDependencies,
      builder: (baseController) {
        Get.log(
          '${this.toString()} start with Controller: ${baseController.runtimeType}',
          isError: false,
        );
        var cachedData = baseController.getData();
        if (cachedData == null || cachedData.length == 0) {
          return FutureBuilder(
            future: baseController.loadData(),
            builder: (context, snapshot) {
              Get.log(
                '${this.toString()} rebuild new data',
                isError: false,
              );
              if (snapshot.connectionState == ConnectionState.waiting) {
                Get.log(
                  '${this.toString()} ConnectionState.waiting return getSkeletonsView',
                  isError: false,
                );
                return this.getSkeletonView(context, baseController);
              } else if (snapshot.hasError) {
                Get.log(
                  '${this.toString()} has error return getErrorView message: ${snapshot.error}',
                  isError: true,
                );
                return this._errorViewAndNoInterNet(snapshot, baseController);
              } else if (snapshot.data != null && snapshot.data.length == 0) {
                Get.log(
                  '${this.toString()} data is empty return getEmptyView',
                  isError: false,
                );
                return this._emptyViewAndNoInterNet(baseController);
              }
              baseController.setData(snapshot.data);
              Get.log(
                '${this.toString()} done process return getMainView',
                isError: false,
              );
              if (Common.supperAdmin && !baseController.isComponent) {
                return _widgetShowMessageID(
                  context,
                  this.getMainView(context, snapshot.data, baseController),
                  baseController,
                );
              }
              return this.getMainView(context, snapshot.data, baseController);
            },
          );
        } else {
          Get.log(
            '${this.toString()} rebuild old data',
            isError: false,
          );
          return this.getMainView(context, cachedData, baseController);
        }
      },
    );
  }

  Widget getWaitingView(BuildContext context) {
    return Common.loadingView ?? LoadingScreen();
  }

  Widget getSkeletonView(BuildContext context, BaseController baseController) {
    String skeleton = baseController.isComponent ? 'component' : 'view';
    Get.log(
      '${this.toString()} build skeleton $skeleton',
      isError: false,
    );
    //show waiting view when is not component
    if (!baseController.isComponent) {
      return getWaitingView(context);
    }
    return Material(
      child: SafeArea(
        child: LayoutBuilder(
          builder: (context, constraints) {
            double heightItem = 150;
            final double height = constraints.maxHeight != double.infinity
                ? constraints.maxHeight
                : heightItem;
            final double width = constraints.maxWidth;
            if (heightItem > height) {
              heightItem = height;
            }
            final int count = height ~/ heightItem;
            return ListView.separated(
              shrinkWrap: true,
              padding: const EdgeInsets.all(16),
              physics: NeverScrollableScrollPhysics(),
              separatorBuilder: (_, __) => SizedBox(height: 16),
              itemCount: count,
              itemBuilder: (_, __) => Container(
                height: heightItem,
                width: width,
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.start,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Expanded(
                      child: Row(
                        children: [
                          Expanded(
                            flex: 2,
                            child: Container(
                              decoration: BoxDecoration(
                                borderRadius: BorderRadius.circular(10),
                                color: Colors.grey[100],
                              ),
                            ),
                          ),
                          SizedBox(width: 8),
                          Expanded(
                            flex: 1,
                            child: Column(
                              children: [
                                Expanded(
                                  child: Container(
                                    decoration: BoxDecoration(
                                      borderRadius: BorderRadius.circular(10),
                                      color: Colors.grey[100],
                                    ),
                                  ),
                                ),
                                SizedBox(height: 8),
                                Expanded(
                                  child: Container(
                                    decoration: BoxDecoration(
                                      borderRadius: BorderRadius.circular(10),
                                      color: Colors.grey[100],
                                    ),
                                  ),
                                ),
                              ],
                            ),
                          )
                        ],
                      ),
                    ),
                    SizedBox(height: 8),
                    Container(
                      height: 20,
                      width: MediaQuery.of(context).size.width,
                      decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(10),
                        color: Colors.grey[100],
                      ),
                    ),
                    SizedBox(height: 8),
                    Container(
                      height: 20,
                      width: MediaQuery.of(context).size.width / 1.5,
                      decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(10),
                        color: Colors.grey[100],
                      ),
                    )
                  ],
                ),
              ),
            );
          },
        ),
      ),
    );
  }

  Widget _errorViewAndNoInterNet(
      AsyncSnapshot<Map<String, List<Map<String, Object>>>> snapshot,
      BaseController baseController) {
    if (baseController.isOnline) {
      baseController.loadData();
      return getErrorView(snapshot, baseController);
    } else {
      return noInternet();
    }
  }

  Widget getErrorView(
    AsyncSnapshot<Map<String, List<Map<String, Object>>>> snapshot,
    BaseController baseController,
  ) {
    return Center(
      child: Image.asset(
        "assets/images/404-error.png",
        height: 100,
        width: 100,
      ),
    );
  }

  Widget _emptyViewAndNoInterNet(BaseController baseController) {
    if (baseController.isOnline) {
      baseController.loadData();
      return getEmptyView(baseController);
    } else {
      return noInternet();
    }
  }

  Widget getEmptyView(BaseController baseController) {
    return Center(
      child: Image.asset(
        "assets/images/emptyFolder.png",
        height: 100,
        width: 100,
      ),
    );
  }

  Widget noInternet() {
    Get.log('${this.runtimeType} no internet', isError: true);
    return Common.widgetNotificationInternet ??
        Center(
          child: Image.asset(
            "assets/images/404-error.png",
            height: 100,
            width: 100,
          ),
        );
  }

  Widget _widgetShowMessageID(
    BuildContext context,
    Widget mainView,
    BaseController baseController,
  ) {
    return Stack(
      children: [
        mainView,
        Positioned(
          right: 10,
          bottom: 50,
          child: Container(
            child: Material(
              borderRadius: BorderRadius.circular(100),
              child: InkWell(
                onTap: () {
                  // Common.showKeyMessage = !Common.showKeyMessage;
                  // baseController.update();
                  showAlertDialog(context, baseController);
                },
                child: Container(
                  width: 50,
                  height: 50,
                  padding: const EdgeInsets.all(8),
                  decoration: BoxDecoration(
                    color: Colors.blue[50],
                    shape: BoxShape.circle,
                  ),
                  child: Center(
                    child: !Common.showKeyMessage
                        ? Icon(Icons.visibility_outlined)
                        : Icon(Icons.visibility_off_outlined),
                  ),
                ),
              ),
            ),
          ),
        )
      ],
    );
  }

  Widget getMainView(
    BuildContext context,
    Map<String, List<Map<String, Object>>> data,
    BaseController controller,
  );

  showAlertDialog(BuildContext context, BaseController baseController) {
    // set up the button
    Widget dsMessageButton = TextButton(
      child: Text("Danh sách message"),
      onPressed: () {
        Get.back();
        Get.to(
          () => ScreenMessage(
            data: baseController.messageIdBackUp,
          ),
        );
      },
    );
    Widget displayMessageIdButton = TextButton(
      child: Text(
        Common.canGetMessage ? "Hiển thị bình thường" : "Hiển thị message ID",
      ),
      onPressed: () {
        Common.canGetMessage = !Common.canGetMessage;
        baseController.update();
        Get.back();
      },
    );
    // set up the AlertDialog
    AlertDialog alert = AlertDialog(
      title: Text("Supper Admin"),
      actions: [
        dsMessageButton,
        displayMessageIdButton,
      ],
    );

    // show the dialog
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return alert;
      },
    );
  }
}
